


import os
import sys
import time

import analytics
from mitoinstaller.create_startup_file import create_startup_file
from mitoinstaller.installer_steps.installer_step import InstallerStep
from mitoinstaller.jupyter_utils import get_prefered_jupyter_env_variable
from mitoinstaller.starter_notebook import (MITO_STARTER_NOTEBOOK_PATH,
                                            try_create_starter_notebook)
from mitoinstaller.user_install import is_running_test
from termcolor import colored  # type: ignore

def replace_process_with_jupyter():
    """
    Switch the currently running process with a running lab or notebook
    instance, depending on which we think the user would prefer.

    If we are running tests, then we do not launch JLab. 
    # TODO: we don't want to launch this if we're inside a Docker script?
    """
    if is_running_test():
        return
    # Get the prefered jupyter to launch, which we saved before
    prefered_jupyter = get_prefered_jupyter_env_variable()

    # Flush analytics before we terminate the process, as it's our last chance
    analytics.flush()

    os.execl(sys.executable, 'python', '-m', 'jupyter', prefered_jupyter, MITO_STARTER_NOTEBOOK_PATH)



def print_success_message():
    """
    Prints a success message to the user, in the case that we cannot
    launch JupyterLab. TODO: Furthermore, logs that the jupyter lab instance
    could not be launched.
    """
    # Finially, if we get here, we print to the user that they should launch
    # JLab and run it. Note that we only get here if the final install step of
    # launching JLab does not work
    print("\n\nThe mitosheet package has correctly been installed.")
    print("To see instructions to render a mitosheet, see docs: ", colored('https://docs.trymito.io/how-to/creating-a-mitosheet', 'green'))


FINAL_INSTALLER_STEPS = [
    InstallerStep(
        'Create import mito startup file',
        create_startup_file,
        optional=True
    ),
    InstallerStep(
        'Creating a Mitosheet starter notebook',
        try_create_starter_notebook
    ),
    InstallerStep(
        'Start Jupyter',
        replace_process_with_jupyter,
        optional=True
    ),
    # We do out best to replace the running process with JupyterLab, but 
    # if this fails, then just print a success message to the user
    InstallerStep(
        'Finish Installation',
        print_success_message,
        optional=True
    )
]


"""
Useful functions and utilities for
logging information about install.

Also utils for printing information
to the user.
"""
import getpass
import platform
import re
import sys
import traceback
from typing import Any, Dict

import analytics

analytics.write_key = '6I7ptc5wcIGC4WZ0N1t0NXvvAbjRGUgX' 

from mitoinstaller import __version__
from mitoinstaller.user_install import (get_mitosheet_telemetry,
                                        get_static_user_id, is_running_test,
                                        user_json_is_installer_default)


def is_on_kuberentes_mito() -> bool:
    """
    Returns True if the user is on Kuberentes Mito, on staging or on app
    """
    user = getpass.getuser()
    return user == 'jovyan'

def is_local_deployment() -> bool:
    """
    Helper function for figuring out if this a local deployment or a
    Mito server deployment
    """
    return not is_on_kuberentes_mito() 

def identify() -> None:
    """
    This identify call identifies the user in our logs. Note that
    we make sure that the actual identify call only runs when
    the user has a user.json with only a static id, as if we
    have more feilds, then the user has already run the mitosheet.sheet
    call and thus been identified there.
    """
    static_user_id = get_static_user_id()
    operating_system = platform.system()

    if user_json_is_installer_default() and not is_running_test():
        analytics.identify(
            static_user_id,
            {
                'operating_system': operating_system,
                'version_installer': __version__,
                'version_sys': sys.version,
                'local': is_local_deployment()
            }
        )

def get_recent_traceback() -> str:
    """
    Helper function that returns the most recent traceback, with the file paths
    stripped to remove all but the mitosheet file names for ease in debugging.
    
    Inspired by https://stackoverflow.com/questions/25272368/hide-file-from-traceback
    """
    return re.sub(r'File ".*[\\/]([^\\/]+.py)"', r'File "\1"', traceback.format_exc())

def log_error(event: str, params: Dict[str, Any]=None) -> None:
    """
    Logs an error by adding the traceback to the error, for easier 
    debugging.
    """
    if params is None:
        params = {}

    recent_traceback = get_recent_traceback().strip().split('\n')
    # Then, we log it
    log(
        event,
        {  
            **params,
            'error_traceback': recent_traceback,
            # We get the last line of the error as it makes it much easier
            # to easily analyze on error messages 
            'error_traceback_last_line': recent_traceback[-1],
        }
    )


def log(event: str, params: Dict[str, Any]=None) -> None:
    """
    A utility that all logging should pass through
    """
    static_user_id = get_static_user_id()

    if params is None:
        params = {}

    # Don't log if telemetry is turned off
    if not is_running_test() and get_mitosheet_telemetry():
        analytics.track(
            static_user_id, 
            event, 
            params
        )



import pytest
from tests.conftest import VirtualEnvironment
import os

def test_install_mitosheet_no_dependencies(venv: VirtualEnvironment):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])
    venv.run_python_module_command(['mitoinstaller', 'install', '--test-pypi'])
    mitosheet_version = venv.get_package_version('mitosheet')
    jlab_version = venv.get_package_version('jupyterlab')

    assert mitosheet_version is not None 
    assert jlab_version.startswith('3')


def test_installs_mitosheet_jlab_3(venv: VirtualEnvironment):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])
    venv.run_python_module_command(['pip', 'install', 'jupyterlab==3.0'])
    
    venv.run_python_module_command(['mitoinstaller', 'install', '--test-pypi'])
    
    mitosheet_version = venv.get_package_version('mitosheet')

    assert mitosheet_version is not None 
    assert 'mitosheet' in venv.get_jupyterlab_extension_names()


def test_installs_mitosheet_jlab_2_no_extensions(venv: VirtualEnvironment):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])
    venv.run_python_module_command(['pip', 'install', 'jupyterlab==2.2.5'])
    
    venv.run_python_module_command(['mitoinstaller', 'install', '--test-pypi'])
    mitosheet_version = venv.get_package_version('mitosheet')
    jlab_version = venv.get_package_version('jupyterlab')

    assert mitosheet_version is not None 
    assert jlab_version.startswith('3')


def test_create_startup_file(venv: VirtualEnvironment):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])
    venv.run_python_module_command(['pip', 'install', 'jupyterlab==3.0'])
    
    out = venv.run_python_module_command(['mitoinstaller', 'install', '--test-pypi'])

    IMPORT_MITOSHEET_FILE_PATH = os.path.join(os.path.expanduser("~"), '.ipython', 'profile_default', 'startup', 'import_mitosheet.py')
    
    assert os.path.exists(IMPORT_MITOSHEET_FILE_PATH) 

def test_installs_from_test_pypi(venv: VirtualEnvironment):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])    
    venv.run_python_module_command(['mitoinstaller', 'install', '--test-pypi'])
    
    mitosheet_version = venv.get_package_version('mitosheet')
    jlab_version = venv.get_package_version('jupyterlab')

    assert mitosheet_version is not None 
    assert jlab_version is not None



from mitoinstaller.installer_steps import (FINAL_INSTALLER_STEPS,
                                           INITIAL_INSTALLER_STEPS,
                                           MITOSHEET_INSTALLER_STEPS,
                                           run_installer_steps)


def do_install() -> None:
    """
    Runs the installer steps to install the `mitosheet` package.

    Notably, the process for installing Mito initially and upgrading Mito are
    identical. As such, we reuse this function to upgrade, just with different
    error and logging messages.
    """
    print("Starting install...")

    # Run the initiall installer steps
    run_installer_steps(INITIAL_INSTALLER_STEPS)

    # Then, we try to install mitosheet package
    run_installer_steps(MITOSHEET_INSTALLER_STEPS)

    # Then, we try and launch JLab
    run_installer_steps(FINAL_INSTALLER_STEPS)
